@@ -2,18 +2,20 @@
 //
 // SPDX-License-Identifier: GPL-3.0-only
 
+use crate::grammar::Disambiguation;
 use crate::grammar::Production;
 use std::hash::Hasher;
 
 /// Internal representation of a grammar rule.
 ///
-/// [GrammarRule] is exposed so you can use its type and its traits
+/// [GrammarRule] is exposed so you can use its type and traits
 /// but normally you create a [GrammarRule]
 /// by using a [GrammarBuilder](crate::grammar::GrammarBuilder).
 #[derive(Clone)]
 pub struct GrammarRule {
-    pub(crate) name:        String,
-    pub(crate) productions: Vec<Production>,
+    pub(crate) name:           String,
+    pub(crate) disambiguation: Option<Disambiguation>,
+    pub(crate) productions:    Vec<Production>,
 }
 
 impl std::fmt::Display for GrammarRule {
