@@ -2,13 +2,13 @@
 //
 // SPDX-License-Identifier: GPL-3.0-only
 
+#![deny(missing_docs)]
 #![deny(rustdoc::bare_urls)]
 #![deny(rustdoc::broken_intra_doc_links)]
 #![deny(rustdoc::invalid_codeblock_attributes)]
 #![deny(rustdoc::invalid_html_tags)]
 #![deny(rustdoc::invalid_rust_codeblocks)]
 #![deny(rustdoc::missing_crate_level_docs)]
-// #![warn(missing_docs)]
 #![deny(rustdoc::private_intra_doc_links)]
 #![deny(rustdoc::private_doc_tests)]
 //! Santiago is a lexing and parsing toolkit for Rust.
@@ -78,16 +78,75 @@
 //! // At this point we can define a grammar for this language:
 //! let grammar = santiago::grammar::GrammarBuilder::new()
 //!     // Map the rule "sum" to the sequence of rules "sum", "plus", and "sum"
-//!     .map_to_rules("sum", &["sum", "plus", "sum"])
+//!     .rule_to_rules("sum", &["sum", "plus", "sum"])
 //!     // Map the rule "sum" to the lexeme "INT"
-//!     .map_to_lexemes("sum", &["INT"])
+//!     .rule_to_lexemes("sum", &["INT"])
 //!     // Map the rule "plus" to the lexeme "PLUS"
-//!     .map_to_lexemes("plus", &["PLUS"])
+//!     .rule_to_lexemes("plus", &["PLUS"])
 //!     .finish();
 //!
 //! // With this we can now parse the input
-//! // and get the [Abstract Syntax Tree](https://en.wikipedia.org/wiki/Abstract_syntax_tree):
-//! let ast = &santiago::parser::parse(&grammar, &lexemes).unwrap()[0];
+//! // and get the [Abstract Syntax Tree](https://en.wikipedia.org/wiki/Abstract_syntax_tree).
+//! //
+//! // Note that since our grammar is ambiguous (for now),
+//! // Santiago will return all possible parses of the input:
+//! let abstract_syntax_trees = &santiago::parser::parse(&grammar, &lexemes).unwrap();
+//!
+//! assert_eq!(abstract_syntax_trees.len(), 2);
+//! assert_eq!(
+//!     vec![
+//!         // Option 1: (11 + 22) + 33
+//!         r#"Γ"#,
+//!         r#"  sum"#,
+//!         r#"    sum"#,
+//!         r#"      sum"#,
+//!         r#"        INT "11" (1, 1)"#,
+//!         r#"      plus"#,
+//!         r#"        PLUS "+" (1, 4)"#,
+//!         r#"      sum"#,
+//!         r#"        INT "22" (1, 6)"#,
+//!         r#"    plus"#,
+//!         r#"      PLUS "+" (1, 9)"#,
+//!         r#"    sum"#,
+//!         r#"      INT "33" (1, 11)"#,
+//!
+//!         // Option 2: 11 + (22 + 33)
+//!         r#"Γ"#,
+//!         r#"  sum"#,
+//!         r#"    sum"#,
+//!         r#"      INT "11" (1, 1)"#,
+//!         r#"    plus"#,
+//!         r#"      PLUS "+" (1, 4)"#,
+//!         r#"    sum"#,
+//!         r#"      sum"#,
+//!         r#"        INT "22" (1, 6)"#,
+//!         r#"      plus"#,
+//!         r#"        PLUS "+" (1, 9)"#,
+//!         r#"      sum"#,
+//!         r#"        INT "33" (1, 11)"#
+//!     ],
+//!     abstract_syntax_trees.iter()
+//!         .map(|ast| ast.to_string())
+//!         .collect::<String>()
+//!         .lines()
+//!         .collect::<Vec<&str>>(),
+//! );
+//!
+//! // Removing ambiguities from a grammar is not a problem.
+//! // Let's add a few disambiguation rules at the end of our grammar:
+//! let grammar = santiago::grammar::GrammarBuilder::new()
+//!     .rule_to_rules("sum", &["sum", "plus", "sum"])
+//!     .rule_to_lexemes("sum", &["INT"])
+//!     .rule_to_lexemes("plus", &["PLUS"])
+//!     // Specify that we want the "plus" rule to be left-associative.
+//!     // In other words: `a+b+c` should group as `(a+b)+c`.
+//!     .disambiguate(santiago::grammar::Associativity::Left, &["plus"])
+//!     .finish();
+//!
+//! // Now parse!
+//! let abstract_syntax_trees = &santiago::parser::parse(&grammar, &lexemes).unwrap();
+//!
+//! assert_eq!(abstract_syntax_trees.len(), 1);
 //! assert_eq!(
 //!     vec![
 //!         r#"Γ"#,
@@ -103,9 +162,12 @@
 //!         r#"      PLUS "+" (1, 9)"#,
 //!         r#"    sum"#,
 //!         r#"      INT "33" (1, 11)"#,
-//!         r#""#
 //!     ],
-//!     ast.to_string().split('\n').collect::<Vec<&str>>(),
+//!     abstract_syntax_trees.iter()
+//!         .map(|ast| ast.to_string())
+//!         .collect::<String>()
+//!         .lines()
+//!         .collect::<Vec<&str>>(),
 //! );
 //! ```
 //!
@@ -144,7 +206,6 @@
 //! ## Example: Smallest lexer possible
 //!
 //! This lexer will copy char by char the input
-//!
 //! ```rust
 //! let input = "abc";
 //!
@@ -169,7 +230,6 @@
 //! ```
 //!
 //! ## Example: JavaScript string interpolations:
-//!
 //! ```rust
 //! let input = "    `a${ variable }b`    ";
 //!
@@ -251,16 +311,15 @@
 //!
 //! For example, a language that recognizes "numbers",
 //! which can be of integer or float type:
-//!
 //! ```rust
 //! santiago::grammar::GrammarBuilder::new()
 //!     // A `number` can be an `int` or a `float`
-//!     .map_to_rules("number", &["int"])
-//!     .map_to_rules("number", &["float"])
+//!     .rule_to_rules("number", &["int"])
+//!     .rule_to_rules("number", &["float"])
 //!     // An `int` comes from a lexeme of kind `INT`:
-//!     .map_to_lexemes("int", &["INT"])
+//!     .rule_to_lexemes("int", &["INT"])
 //!     // An `float` comes from a lexeme of kind `FLOAT`:
-//!     .map_to_lexemes("float", &["FLOAT"])
+//!     .rule_to_lexemes("float", &["FLOAT"])
 //!     .finish();
 //! ```
 //!
@@ -268,10 +327,11 @@
 //! For instance, the following example references a rule
 //! that is not defined later in the grammar,
 //! which constitutes an error:
-//!
 //! ```should_panic
 //! // Map rule `A` to `B` and don't define rule B:
-//! santiago::grammar::GrammarBuilder::new().map_to_rules("A", &["B"]).finish();
+//! santiago::grammar::GrammarBuilder::new()
+//!     .rule_to_rules("A", &["B"])
+//!     .finish();
 //! ```
 pub mod grammar;
 pub mod languages;
@@ -282,12 +342,12 @@ pub mod parser;
 ///
 /// # Example
 ///
-/// Reuse regular expressions
+/// Reuse regular expressions.
 ///
 /// ```rust
-/// santiago::def!(INT, r"\d+"); // 1 or more digits
-/// santiago::def!(SIGN, r"[+-]?"); // either "+" or "-", optional
-/// santiago::def!(SIGNED_INT, concat!(SIGN!(), INT!())); // A sign, then an integer
+/// santiago::def!(INT, r"\d+"); // 1 or more digits.
+/// santiago::def!(SIGN, r"[+-]?"); // either "+" or "-", optional.
+/// santiago::def!(SIGNED_INT, concat!(SIGN!(), INT!())); // A sign, then an integer.
 ///
 /// assert_eq!(SIGNED_INT!(), r"[+-]?\d+");
 /// ```
